/*
 * Copyright (C) 2003-2004 Red Hat Inc. All Rights Reserved.
 *
 * The contents of this file are subject to the CCM Public
 * License (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the
 * License at http://www.redhat.com/licenses/ccmpl.html.
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language
 * governing rights and limitations under the License.
 *
 */
package com.arsdigita.versioning;

import java.math.BigInteger;

/**
 * A sequence generator. Use this instead of a DB-backed sequence to ensure that
 * the same primary key values are used in any two independent runs of the same
 * unit test suite.
 *
 * @author Vadim Nasardinov (vadimn@redhat.com)
 * @since 2003-02-26
 * @version $Revision: #4 $ $DateTime: 2004/04/07 16:07:11 $
 **/
final class Sequence {
    private static BigInteger s_current = BigInteger.ZERO;

    synchronized public static BigInteger next() {
        s_current = s_current.add(BigInteger.ONE);
        return s_current;
    }
}
